@@ -11,7 +11,7 @@
  * This file contains all the "glue code" required to use the canon
  * driver with libgphoto2.
  *
- * $Id: library.c,v 1.80 2003/09/03 22:36:26 hun Exp $
+ * $Id: library.c,v 1.80.4.1 2004/01/14 18:33:25 hun Exp $
  *
  ****************************************************************************/
 
@@ -131,9 +131,8 @@ camera_manual (Camera *camera, CameraText *manual, GPContext *context)
 	GP_DEBUG ("camera_manual()");
 
 	strcpy (manual->text,
-		_("For the A50, 115200 may not be faster than 57600\n"
-		  "Folders are NOT supported\n"
-		  "if you experience a lot of serial transmission errors,"
+		_("For the A50, using 115200 may effectively be slower than using 57600\n"
+		  "If you experience a lot of serial transmission errors,"
 		  " try to have your computer as idle as possible\n"
 		  " (i.e. no disk activity)\n"
 			));
